@@ -2,6 +2,7 @@ import mimetypes
 import os
 import re
 import requests
+from libthumbor import CryptoURL
 from requests.packages.urllib3.exceptions import LocationParseError
 from StringIO import StringIO
 from django.conf import settings
@@ -56,7 +57,7 @@ class ThumborStorageFile(ImageFile):
         self._file = value
 
     file = property(_get_file, _set_file)
-    
+
     @property
     def size(self):
         self.seek(0, os.SEEK_END)
@@ -88,8 +89,8 @@ class ThumborStorage(Storage):
 
     def exists(self, name):
         # name is the location returned by Thumbor when posted > may exists.
-        if re.match(r"^/image/\w{32}/.*$", name):
-            return thumbor_original_exists(thumbor_image_url(name))
+        if re.match(r"^/image/\w{32}(?:/.*){0,1}$", name):
+            return thumbor_original_exists(thumbor_original_image_url(name))
         # name as defined in 'upload_to' > new image.
         else:
             return False
@@ -99,10 +100,10 @@ class ThumborStorage(Storage):
         return f.size
 
     def url(self, name):
-        return thumbor_image_url(name)
+        return thumbor_image_url(self.key(name))
 
     def key(self, name):
-        return re.match(r"^/image/(?P<key>\w{32})/.*$", name).groupdict()['key']
+        return re.match(r"^/image/(?P<key>\w{32})(?:/.*){0,1}$", name).groupdict()['key']
 
     def get_available_name(self, name):
         # There is no way to know if the image exists on Thumbor.
@@ -138,35 +139,44 @@ class ThumborMigrationStorage(ThumborStorage, FileSystemStorage):
         FileSystemStorage.__init__(self, location=location, base_url=base_url)
 
     def _open(self, name, mode='rb'):
-        if re.match(r"^/image/\w{32}/.*$", name):
+        if self.is_thumbor(name):
             return ThumborStorage._open(self, name, mode)
         else:
             return FileSystemStorage._open(self, name, mode)
 
     def delete(self, name):
-        if re.match(r"^/image/\w{32}/.*$", name):
+        if self.is_thumbor(name):
             return ThumborStorage.delete(self, name)
         else:
             return FileSystemStorage.delete(self, name)
 
+    def exists(self, name):
+        if self.is_thumbor(name):
+            return ThumborStorage.exists(self, name)
+        else:
+            return FileSystemStorage.exists(self, name)
+
     def url(self, name):
-        if re.match(r"^/image/\w{32}/.*$", name):
+        if self.is_thumbor(name):
             return ThumborStorage.url(self, name)
         else:
             return FileSystemStorage.url(self, name)
 
     def key(self, name):
-        if re.match(r"^/image/\w{32}/.*$", name):
+        if self.is_thumbor(name):
             return ThumborStorage.key(self, name)
         else:
             raise NotImplementedError
 
     def path(self, name):
-        if re.match(r"^/image/\w{32}/.*$", name):
+        if self.is_thumbor(name):
             return ThumborStorage.path(self, name)
         else:
             return FileSystemStorage.path(self, name)
 
+    def is_thumbor(self, name):
+        return re.match(r"^/image/\w{32}(?:/.*){0,1}$", name)
+
 
 def thumbor_original_exists(url):
     # May be cool to be able to check if the image exists on Thumbor server
@@ -174,7 +184,7 @@ def thumbor_original_exists(url):
     try:
         response = requests.get(url)
     # Happens when trying to get an image when the name in db 
-    # is in a FileSystemStorage form (without the '/' at the beginning).
+    # is in a FileSystemStorage form (without the leading slash).
     except LocationParseError:
         return False
     if response.status_code == 200:
@@ -187,5 +197,9 @@ def thumbor_original_exists(url):
 #   self being a ThumborMigrationStorage instance and result in infinite loop.
 # These methods proxiing to these functions.
 
-def thumbor_image_url(name):
+def thumbor_image_url(key):
+    crypto = CryptoURL(key=settings.THUMBOR_SECURITY_KEY)
+    return "%s%s" % (settings.THUMBOR_SERVER, crypto.generate(image_url=key))
+
+def thumbor_original_image_url(name):
     return "%s%s" % (settings.THUMBOR_RW_SERVER, name)
