@@ -13,9 +13,13 @@ class MockedGetResponse:
 
     def __init__(self, url):
         """Retrieve the file on the filesytem according to the name. """
+        #TODO: catch LocationParseError.
         basename = os.path.basename(url)
         filename = os.path.join(IMAGE_DIR, basename)
-        self.content = open(filename, "r").read()
+        if not os.path.exists(filename):
+            self.status_code = 404
+        else:
+            self.content = open(filename, "r").read()
 
 
 def mocked_thumbor_get_response(url):
@@ -72,6 +76,7 @@ class DjangoThumborTestCase(unittest.TestCase):
 
         self.patcher_delete = mock.patch('django_thumborstorage.storages.requests.delete')
         self.MockDeleteClass = self.patcher_delete.start()
+        self.MockDeleteClass.side_effect = mocked_thumbor_delete_allowed_response
 
     def tearDown(self):
         self.patcher_get.stop()
@@ -97,6 +102,15 @@ class ThumborStorageFileTest(DjangoThumborTestCase):
         self.MockGetClass.assert_called_with("%s%s" % (settings.THUMBOR_RW_SERVER, filename))
         self.assertEqual(size, 9730)
 
+    def test_read(self):
+        from django_thumborstorage import storages
+        from django.conf import settings
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
+        thumbor_file = storages.ThumborStorageFile(filename, mode='r')
+        content = thumbor_file.read()
+        self.MockGetClass.assert_called_with("%s%s" % (settings.THUMBOR_RW_SERVER, filename))
+        self.assertEqual(len(content), 9730)
+
     def test_write_jpeg(self):
         from django_thumborstorage import storages
         from django.conf import settings
@@ -122,7 +136,6 @@ class ThumborStorageFileTest(DjangoThumborTestCase):
         self.assertEqual(thumbor_file._location, '/image/oooooo32chars_random_idooooooooo/%s' % filename)
 
     def test_delete_allowed(self):
-        self.MockDeleteClass.side_effect = mocked_thumbor_delete_allowed_response
         from django_thumborstorage import storages
         from django.conf import settings
         filename = '/image/oooooo32chars_random_idooooooooo/foundations/gnu.png'
@@ -159,13 +172,16 @@ class ThumborStorageTest(DjangoThumborTestCase):
         from django.conf import settings
         filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
         self.assertEqual(self.storage.url(filename),
-                         '%s/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg' % settings.THUMBOR_RW_SERVER)
+                         '%s/qn6d7XNEzldMxgE8t4oVjEbEsDg=/5247a82854384f228c6fba432c67e6a8' % settings.THUMBOR_SERVER)
 
     def test_key(self):
         from django.conf import settings
         filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
         self.assertEqual(self.storage.key(filename), '5247a82854384f228c6fba432c67e6a8')
 
+        filename = '/image/5247a82854384f228c6fba432c67e6a8'
+        self.assertEqual(self.storage.key(filename), '5247a82854384f228c6fba432c67e6a8')
+
     def test_size(self):
         from django.conf import settings
         filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
@@ -183,6 +199,22 @@ class ThumborStorageTest(DjangoThumborTestCase):
                                               headers={"Content-Type": "image/jpeg", "Slug": filename})
         self.assertEqual(response, '/image/oooooo32chars_random_idooooooooo/%s' % filename)
 
+    def test_delete(self):
+        from django.conf import settings
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
+        self.storage.delete(filename)
+        self.MockDeleteClass.assert_called_with("%s%s" % (settings.THUMBOR_RW_SERVER, filename))
+
+    def test_exists(self):
+        from django.conf import settings
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
+        self.assertTrue(self.storage.exists(filename))
+        self.MockGetClass.assert_called_with("%s%s" % (settings.THUMBOR_RW_SERVER, filename))
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/DoesNoyExists.jpg'
+        self.assertFalse(self.storage.exists(filename))
+        filename = 'people/new/TempletonPeck.jpg'
+        self.assertFalse(self.storage.exists(filename))
+
 
 class ThumborMigrationStorageTest(DjangoThumborTestCase):
     def setUp(self):
@@ -190,17 +222,41 @@ class ThumborMigrationStorageTest(DjangoThumborTestCase):
         from django_thumborstorage import storages
         self.storage = storages.ThumborMigrationStorage()
 
+    def test_is_thumbor(self):
+        self.assertTrue(self.storage.is_thumbor('/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'))
+        self.assertTrue(self.storage.is_thumbor('/image/5247a82854384f228c6fba432c67e6a8'))
+        self.assertFalse(self.storage.is_thumbor('images/people/new/TempletonPeck.jpg'))
+        self.assertFalse(self.storage.is_thumbor('/image/5247a82854384f228c6fba432c67e6a8BlahBlahBlah'))
+
     def test_url_thumbor(self):
         from django.conf import settings
         filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
         self.assertEqual(self.storage.url(filename),
-                         '%s/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg' % settings.THUMBOR_RW_SERVER)
+                         '%s/qn6d7XNEzldMxgE8t4oVjEbEsDg=/5247a82854384f228c6fba432c67e6a8' % settings.THUMBOR_SERVER)
 
     def test_key_thumbor(self):
         from django.conf import settings
         filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
         self.assertEqual(self.storage.key(filename), '5247a82854384f228c6fba432c67e6a8')
 
+    def test_path_thumbor(self):
+        from django.conf import settings
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
+        with self.assertRaises(NotImplementedError):
+            self.storage.path(filename)
+
+    def test_delete_thumbor(self):
+        from django.conf import settings
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
+        self.storage.delete(filename)
+        self.MockDeleteClass.assert_called_with("%s%s" % (settings.THUMBOR_RW_SERVER, filename))
+
+    def test_exists_thumbor(self):
+        from django.conf import settings
+        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
+        self.assertTrue(self.storage.exists(filename))
+        self.MockGetClass.assert_called_with("%s%s" % (settings.THUMBOR_RW_SERVER, filename))
+
     def test_url_filesystem(self):
         from django.conf import settings
         filename = 'images/people/new/TempletonPeck.jpg'
@@ -213,17 +269,24 @@ class ThumborMigrationStorageTest(DjangoThumborTestCase):
         with self.assertRaises(NotImplementedError):
             self.storage.key(filename)
 
-    def test_path_thumbor(self):
-        from django.conf import settings
-        filename = '/image/5247a82854384f228c6fba432c67e6a8/people/new/TempletonPeck.jpg'
-        with self.assertRaises(NotImplementedError):
-            self.storage.path(filename)
-
     def test_path_filesystem(self):
         from django.conf import settings
         filename = 'images/people/new/TempletonPeck.jpg'
         self.assertEqual(self.storage.path(filename), '/media/images/people/new/TempletonPeck.jpg')
 
+    def test_delete_filesytem(self):
+        from django.conf import settings
+        filename = 'images/people/new/TempletonPeck.jpg'
+        # Note: FileSystemStorage does not raises exception if the file does not exists.
+        self.storage.delete(filename)
+        assert not self.MockDeleteClass.called, "Should not DELETE on Thumbor."
+
+    def test_exists_filesystem(self):
+        from django.conf import settings
+        filename = 'images/people/new/TempletonPeck.jpg'
+        self.storage.exists(filename)
+        assert not self.MockGetClass.called, "Should not GET on Thumbor."
+
 
 def suite():
     suite = unittest.TestLoader().loadTestsFromTestCase(ThumborStorageTest)
